@@ -1,24 +1,40 @@
 #ifndef lint
-static char *RCSid = "$Id: wtext.c%v 3.50.1.13 1993/08/19 03:21:26 woo Exp $";
+static char *RCSid = "$Id: wtext.c,v 1.3.2.1 1999/09/21 18:48:28 lhecking Exp $";
 #endif
 
 /* GNUPLOT - win/wtext.c */
-/*
- * Copyright (C) 1992   Russell Lang
+
+/*[
+ * Copyright 1992, 1993, 1998   Russell Lang
  *
  * Permission to use, copy, and distribute this software and its
- * documentation for any purpose with or without fee is hereby granted, 
- * provided that the above copyright notice appear in all copies and 
- * that both that copyright notice and this permission notice appear 
+ * documentation for any purpose with or without fee is hereby granted,
+ * provided that the above copyright notice appear in all copies and
+ * that both that copyright notice and this permission notice appear
  * in supporting documentation.
  *
  * Permission to modify the software is granted, but not the right to
- * distribute the modified code.  Modifications are to be distributed 
- * as patches to released version.
- *  
- * This software is provided "as is" without express or implied warranty.
- * 
+ * distribute the complete modified source code.  Modifications are to
+ * be distributed as patches to the released version.  Permission to
+ * distribute binaries produced by compiling modified sources is granted,
+ * provided you
+ *   1. distribute the corresponding source modifications from the
+ *    released version in the form of a patch file along with the binaries,
+ *   2. add special version identification to distinguish your version
+ *    in addition to the base release version number,
+ *   3. provide your name and address as the primary contact for the
+ *    support of your modified version, and
+ *   4. retain our contact information in regard to use of the base
+ *    software.
+ * Permission to distribute the released version of the source code along
+ * with corresponding source modifications in the form of a patch file is
+ * granted with same provisions 2 through 4 for binary distributions.
  *
+ * This software is provided "as is" without express or implied warranty
+ * to the extent permitted by applicable law.
+]*/
+
+/*
  * AUTHORS
  * 
  *   Russell Lang
@@ -26,7 +42,7 @@ static char *RCSid = "$Id: wtext.c%v 3.50.1.13 1993/08/19 03:21:26 woo Exp $";
  * Send your comments or suggestions to 
  *  info-gnuplot@dartmouth.edu.
  * This is a mailing list; to join it send a note to 
- *  info-gnuplot-request@dartmouth.edu.  
+ *  majordomo@dartmouth.edu.  
  * Send bug reports to
  *  bug-gnuplot@dartmouth.edu.
  */
@@ -35,18 +51,20 @@ static char *RCSid = "$Id: wtext.c%v 3.50.1.13 1993/08/19 03:21:26 woo Exp $";
    in win/wtext.h */
 
 #define STRICT
-#include <windows.h>
-#include <windowsx.h>
-#if WINVER >= 0x030a
-#include <commdlg.h>
-#endif
+
 #include <string.h>	/* use only far items */
-#ifndef __MSC__
-#include <mem.h>
-#endif
 #include <stdlib.h>
 #include <ctype.h>
 #include <dos.h>
+#ifndef __MSC__
+# include <mem.h>
+#endif
+
+#include <windows.h>
+#include <windowsx.h>
+#if WINVER >= 0x030a
+# include <commdlg.h>
+#endif
 
 #include "wgnuplib.h"
 #include "wresourc.h"
@@ -56,12 +74,16 @@ static char *RCSid = "$Id: wtext.c%v 3.50.1.13 1993/08/19 03:21:26 woo Exp $";
 #define TEXTFONTSIZE 9
 #define TEXTFONTNAME "Terminal"
 
+#ifndef EOF /* HBB 980809: for MinGW32 */
 #define EOF -1		/* instead of using <stdio.h> */
+#endif
 /* limits */
 #define MAXSTR 256
 POINT ScreenMinSize = {16,4};
-LRESULT CALLBACK _export WndParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
-LRESULT CALLBACK _export WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
+
+LRESULT CALLBACK WINEXPORT WndParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
+LRESULT CALLBACK WINEXPORT WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
+
 void ReadTextIni(LPTW lptw);
 void LimitMark(LPTW lptw, POINT FAR *lppt);
 
@@ -98,6 +120,10 @@ TextMessage(void)
 
     while (PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
         {
+#if 1 /* HBB 19990505: Petzold says we should check this: */
+        if (msg.message == WM_QUIT)
+            return;
+#endif
         TranslateMessage(&msg);
         DispatchMessage(&msg);
         }
@@ -110,6 +136,9 @@ void
 CreateTextClass(LPTW lptw)
 {
 WNDCLASS wndclass;
+#ifdef WIN32
+hdllInstance = lptw->hInstance;	/* not using a DLL */
+#endif
 	wndclass.style = CS_HREDRAW | CS_VREDRAW;
 	wndclass.lpfnWndProc = WndTextProc;
 	wndclass.cbClsExtra = 0;
@@ -134,7 +163,7 @@ WNDCLASS wndclass;
 	else
 		wndclass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
 	wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
-	wndclass.hbrBackground = GetStockBrush(WHITE_BRUSH);
+	wndclass.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
 	wndclass.lpszMenuName = NULL;
 	wndclass.lpszClassName = szParentClass;
 	RegisterClass(&wndclass);
@@ -203,7 +232,6 @@ TextInit(LPTW lptw)
 		MessageBox((HWND)NULL,"Couldn't open parent text window",(LPSTR)NULL, MB_ICONHAND | MB_SYSTEMMODAL);
 		return(1);
 	}
-	ShowWindow(lptw->hWndParent, lptw->nCmdShow);
 	GetClientRect(lptw->hWndParent, &rect);
 
 	lptw->hWndText = CreateWindow(szTextClass, lptw->Title,
@@ -322,15 +350,15 @@ ReadTextIni(LPTW lptw)
 
 	if (bOKINI)
 	  GetPrivateProfileString(section, "TextOrigin", "", profile, 80, file);
-	if ( (p = GetInt(profile, &lptw->Origin.x)) == NULL)
+	if ( (p = GetInt(profile, (LPINT)&lptw->Origin.x)) == NULL)
 		lptw->Origin.x = CW_USEDEFAULT;
-	if ( (p = GetInt(p, &lptw->Origin.y)) == NULL)
+	if ( (p = GetInt(p, (LPINT)&lptw->Origin.y)) == NULL)
 		lptw->Origin.y = CW_USEDEFAULT;
 	if ( (file != (LPSTR)NULL) && (section != (LPSTR)NULL) )
 	  GetPrivateProfileString(section, "TextSize", "", profile, 80, file);
-	if ( (p = GetInt(profile, &lptw->Size.x)) == NULL)
+	if ( (p = GetInt(profile, (LPINT)&lptw->Size.x)) == NULL)
 		lptw->Size.x = CW_USEDEFAULT;
-	if ( (p = GetInt(p, &lptw->Size.y)) == NULL)
+	if ( (p = GetInt(p, (LPINT)&lptw->Size.y)) == NULL)
 		lptw->Size.y = CW_USEDEFAULT;
 
 	if (bOKINI)
@@ -437,7 +465,7 @@ int xpos, ypos;
 	    SetTextColor(hdc, TextFore(lptw->Attr));
 	    SetBkColor(hdc, TextBack(lptw->Attr));
 	}
-	SelectFont(hdc, lptw->hfont);
+	SelectObject(hdc, lptw->hfont);
 	TextOut(hdc,xpos,ypos,
 		(LPSTR)(lptw->ScreenBuffer + lptw->CursorPos.y*lptw->ScreenSize.x + 
 		lptw->CursorPos.x), count);
@@ -461,7 +489,7 @@ int pos;
 			NewLine(lptw);
 			break;
 		case 7:
-			MessageBeep(-1);
+			MessageBeep(0xFFFFFFFF);
 			if (lptw->CursorFlag)
 				TextToCursor(lptw);
 			break;
@@ -636,7 +664,7 @@ int count;
 int offset;
 	offset = lptw->ScreenSize.x * pt.y + pt.x;
 	hdc = GetDC(lptw->hWndText);
-	SelectFont(hdc, lptw->hfont);
+	SelectObject(hdc, lptw->hfont);
 	if (lptw->bSysColors) {
 	    SetTextColor(hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
 	    SetBkColor(hdc, GetSysColor(COLOR_HIGHLIGHT));
@@ -806,7 +834,7 @@ TextCopyClip(LPTW lptw)
 	hGMem = GlobalReAlloc(hGMem, (DWORD)size, GMEM_MOVEABLE);
 	/* find out what type to put into clipboard */
 	hdc = GetDC(lptw->hWndText);
-	SelectFont(hdc, lptw->hfont);
+	SelectObject(hdc, lptw->hfont);
 	GetTextMetrics(hdc,(TEXTMETRIC FAR *)&tm);
 	if (tm.tmCharSet == OEM_CHARSET)
 		type = CF_OEMTEXT;
@@ -843,10 +871,10 @@ TextMakeFont(LPTW lptw)
 		lf.lfWeight = FW_BOLD;
 	}
 	if (lptw->hfont != 0)
-		DeleteFont(lptw->hfont);
+		DeleteObject(lptw->hfont);
 	lptw->hfont = CreateFontIndirect((LOGFONT FAR *)&lf);
 	/* get text size */
-	SelectFont(hdc, lptw->hfont);
+	SelectObject(hdc, lptw->hfont);
 	GetTextMetrics(hdc,(TEXTMETRIC FAR *)&tm);
 	lptw->CharSize.y = tm.tmHeight;
 	lptw->CharSize.x = tm.tmAveCharWidth;
@@ -912,7 +940,7 @@ TextSelectFont(LPTW lptw) {
 
 
 /* parent overlapped window */
-LRESULT CALLBACK _export
+LRESULT CALLBACK WINEXPORT
 WndParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
 	HDC hdc;
@@ -943,10 +971,10 @@ WndParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 			break;
 		case WM_GETMINMAXINFO:
 			{
-			POINT far * MMinfo = (POINT far *)lParam;
+                        POINT FAR * MMinfo = (POINT FAR *)lParam;
 			TEXTMETRIC tm;
 			hdc = GetDC(hwnd);
-			SelectFont(hdc, GetStockFont(OEM_FIXED_FONT));
+			SelectObject(hdc, GetStockObject(OEM_FIXED_FONT));
 			GetTextMetrics(hdc,(LPTEXTMETRIC)&tm);
 			ReleaseDC(hwnd,hdc);
 			/* minimum size */
@@ -976,8 +1004,8 @@ WndParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 				hbrush = CreateSolidBrush(GetSysColor(COLOR_BTNSHADOW));
 				rect.bottom = lptw->ButtonHeight-1;
 				FillRect(hdc, &rect, hbrush);
-				DeleteBrush(hbrush);
-				SelectPen(hdc, GetStockPen(BLACK_PEN));
+				DeleteObject(hbrush);
+				SelectObject(hdc, GetStockObject(BLACK_PEN));
 				MoveTo(hdc, rect.left, lptw->ButtonHeight-1);
 				LineTo(hdc, rect.right, lptw->ButtonHeight-1);
 			}
@@ -1003,19 +1031,26 @@ WndParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 			/* get character size */
 			TextMakeFont(lptw);
 			hdc = GetDC(hwnd);
-			SelectFont(hdc, lptw->hfont);
+			SelectObject(hdc, lptw->hfont);
 			GetTextMetrics(hdc,(LPTEXTMETRIC)&tm);
 			lptw->CharSize.y = tm.tmHeight;
 			lptw->CharSize.x = tm.tmAveCharWidth;
 			lptw->CharAscent = tm.tmAscent;
 			ReleaseDC(hwnd,hdc);
 			GetClientRect(hwnd, &crect);
-			if ( (lptw->CharSize.y*lptw->ScreenSize.y < crect.bottom)
-			  || (lptw->CharSize.x*lptw->ScreenSize.x < crect.right) ) {
-				/* shrink size */
+			if ( lptw->CharSize.x*lptw->ScreenSize.x < crect.right ) {
+				/* shrink x size */
 			    GetWindowRect(lptw->hWndParent,&wrect);
 			    MoveWindow(lptw->hWndParent, wrect.left, wrect.top,
 				 wrect.right-wrect.left + (lptw->CharSize.x*lptw->ScreenSize.x - crect.right),
+				 wrect.bottom-wrect.top,
+				 TRUE);
+		    }
+			if ( lptw->CharSize.y*lptw->ScreenSize.y < crect.bottom ) {
+				/* shrink y size */
+			    GetWindowRect(lptw->hWndParent,&wrect);
+			    MoveWindow(lptw->hWndParent, wrect.left, wrect.top,
+				 wrect.right-wrect.left,
 				 wrect.bottom-wrect.top + (lptw->CharSize.y*lptw->ScreenSize.y+lptw->ButtonHeight - crect.bottom),
 				 TRUE);
 			}
@@ -1037,7 +1072,7 @@ WndParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 				DragAcceptFiles(hwnd, FALSE);
 			}
 #endif
-			DeleteFont(lptw->hfont);
+			DeleteObject(lptw->hfont);
 			lptw->hfont = 0;
 			break;
 		case WM_CLOSE:
@@ -1051,7 +1086,7 @@ WndParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 }
 
 /* child text window */
-LRESULT CALLBACK _export
+LRESULT CALLBACK WINEXPORT
 WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
 	HDC hdc;
@@ -1347,7 +1382,7 @@ WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 					UINT type;
 					/* find out what type to get from clipboard */
 					hdc = GetDC(hwnd);
-					SelectFont(hdc, lptw->hfont);
+					SelectObject(hdc, lptw->hfont);
 					GetTextMetrics(hdc,(TEXTMETRIC FAR *)&tm);
 					if (tm.tmCharSet == OEM_CHARSET)
 						type = CF_OEMTEXT;
@@ -1391,7 +1426,7 @@ WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 			}
 			return(0);
 		case WM_SYSCOLORCHANGE:
-			DeleteBrush(lptw->hbrBackground);
+			DeleteObject(lptw->hbrBackground);
 			lptw->hbrBackground = CreateSolidBrush(lptw->bSysColors ? 
 				GetSysColor(COLOR_WINDOW) : RGB(0,0,0));
 			return(0);
@@ -1404,7 +1439,7 @@ WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 			hdc = BeginPaint(hwnd, &ps);
 			if (ps.fErase)
 				FillRect(hdc, &ps.rcPaint, lptw->hbrBackground);
-			SelectFont(hdc, lptw->hfont);
+			SelectObject(hdc, lptw->hfont);
 			SetMapMode(hdc, MM_TEXT);
 			SetBkMode(hdc,OPAQUE);
 			GetClientRect(hwnd, &rect);
@@ -1495,7 +1530,7 @@ WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 			lptw->hWndText = hwnd;
 			break;
 		case WM_DESTROY:
-			DeleteBrush(lptw->hbrBackground);
+			DeleteObject(lptw->hbrBackground);
 			break;
 	}
 	return DefWindowProc(hwnd, message, wParam, lParam);
@@ -1526,9 +1561,12 @@ TextGetCh(LPTW lptw)
 			- lptw->CaretHeight - lptw->ScrollPos.y);
 		ShowCaret(lptw->hWndText);
 	}
-	do {
-		TextMessage();
-	} while (!TextKBHit(lptw));
+        while (!TextKBHit(lptw)) { /* CMW: can't use TextMessage here as it does not idle properly */
+                MSG msg;
+                GetMessage(&msg, 0, 0, 0);
+                TranslateMessage(&msg);
+                DispatchMessage(&msg);
+        }
 	ch = *lptw->KeyBufOut++;
 	if (ch=='\r')
 		ch = '\n';
@@ -1557,11 +1595,11 @@ TextGetS(LPTW lptw, LPSTR str, unsigned int size)
 	while (--size>0) {
 		switch(*next = TextGetChE(lptw)) {
 			case EOF:
-				*next = NULL;
+				*next = 0;
 				if (next == str) return (char *)NULL;
 				return str;
 			case '\n':
-				*(next+1) = NULL;
+				*(next+1) = 0;
 				return str;
 			case 0x08:
 			case 0x7f:
@@ -1572,7 +1610,7 @@ TextGetS(LPTW lptw, LPSTR str, unsigned int size)
 				++next;
 		}
 	}
-	*next = NULL;
+	*next = 0;
 	return str;
 }
 
@@ -1637,7 +1675,7 @@ POINT pt;
 	    SetTextColor(hdc, TextFore(lptw->Attr));
 	    SetBkColor(hdc, TextBack(lptw->Attr));
 	}
-	SelectFont(hdc, (lptw->hfont));
+	SelectObject(hdc, (lptw->hfont));
 	TextOut(hdc,xpos,ypos,
 		(LPSTR)(lptw->ScreenBuffer + lptw->CursorPos.y*lptw->ScreenSize.x + 
 		lptw->CursorPos.x), lptw->ScreenSize.x-lptw->CursorPos.x);
@@ -1737,7 +1775,7 @@ TextAttr(LPTW lptw, BYTE attr)
 }
 
 /* About Box */
-BOOL CALLBACK _export
+BOOL CALLBACK WINEXPORT
 AboutDlgProc(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam)
 {
 	switch (wMsg) {
@@ -1779,6 +1817,9 @@ AboutDlgProc(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam)
 void WDPROC
 AboutBox(HWND hwnd, LPSTR str)
 {
+#ifdef WIN32
+	DialogBoxParam(hdllInstance,"AboutDlgBox",hwnd,AboutDlgProc,(LPARAM)str);
+#else
 DLGPROC lpfnAboutDlgProc;
 #ifdef __DLL__
 	lpfnAboutDlgProc = (DLGPROC)GetProcAddress(hdllInstance, "AboutDlgProc");
@@ -1790,4 +1831,5 @@ DLGPROC lpfnAboutDlgProc;
 #ifndef __DLL__
 	FreeProcInstance((FARPROC)lpfnAboutDlgProc);
 #endif
+#endif
 }
